# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module including an extension to the Insights Core Messaging ConsumerWatcher class."""

from insights_messaging.watchers import ConsumerWatcher as ICMConsumerWatcher


class ConsumerWatcher(ICMConsumerWatcher):
    """Extension of the ConsumerWatcher class to allow handling new callbacks."""

    def on_not_handled(self, input_msg):
        """Handle a fired event after a received element cannot be handled by the consumer."""
