# Copyright 2022 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helper class for testing string handling."""


class UnicodeEncodeErrorThrower:
    """UnicodeErrorThrower is helper class for testing string handling."""

    def __init__(self):
        """Initialize new constructed instance of UnicodeErrorThrower."""
        pass

    def encode(self, encoding):
        """Simulate method encode of type str."""
        raise UnicodeEncodeError(encoding, "can not encode", 0, 1, "can not encode")
